@@ -7,7 +7,7 @@ from bson.objectid import ObjectId
 from jsonclasses.jsonclass_object import JSONClassObject
 if TYPE_CHECKING:
     from .dbconf import DBConf
-    from .query import (ListQuery, IDQuery, SingleQuery, ExistQuery,
+    from .query import (BaseQuery, ListQuery, IDQuery, SingleQuery, ExistQuery,
                         IterateQuery)
 
 
@@ -36,6 +36,10 @@ class PymongoObject(JSONClassObject):
     def id(cls: type[T], id: Union[str, ObjectId]) -> IDQuery[T]:
         ...
 
+    @classmethod
+    def linked(cls: type[T]) -> BaseQuery[T]:
+        ...
+
     @classmethod
     def exist(cls: type[T], **kwargs: Any) -> ExistQuery[T]:
         ...
