@@ -7,6 +7,8 @@ from tests.classes.simple_song import SimpleSong
 from tests.classes.simple_artist import SimpleArtist
 from tests.classes.linked_author import LinkedAuthor
 from tests.classes.linked_post import LinkedPost
+from tests.classes.linked_profile_user import LinkedProfile, LinkedUser
+from tests.classes.linked_favorite import LinkedCourse, LinkedStudent
 
 
 class TestSave(TestCase):
@@ -36,6 +38,18 @@ class TestSave(TestCase):
         collection.delete_many({})
         collection = Connection.get_collection(LinkedPost)
         collection.delete_many({})
+        collection = Connection.get_collection(LinkedProfile)
+        collection.delete_many({})
+        collection = Connection.get_collection(LinkedUser)
+        collection.delete_many({})
+        collection.delete_many({})
+        collection = Connection.get_collection(LinkedStudent)
+        collection.delete_many({})
+        collection = Connection.get_collection(LinkedCourse)
+        collection.delete_many({})
+        collection = Connection('linked').collection('linkedcoursesstudents'
+                                                     'linkedstudentscourses')
+        collection.delete_many({})
 
     def test_object_is_saved_into_database(self):
         song = SimpleSong(name='Long', year=2020, artist='Thao')
@@ -199,13 +213,107 @@ class TestSave(TestCase):
         for item in collection.find({}):
             self.assertEqual(item['authorId'], ObjectId(a2.id))
 
-# @jsonclass
-# class Product(MongoObject):
-#     name: str
-#     customers: List[Customer] = types.listof('Customer').linkedthru('products')
+    def test_local_key_from_param_is_saved(self):
+        a1 = LinkedAuthor(name='A1')
+        a1.save()
+        post = LinkedPost(title='P1', content='P2', authorId=a1.id)
+        post.save()
+        collection = Connection.get_collection(LinkedPost)
+        for item in collection.find({}):
+            self.assertEqual(item['authorId'], ObjectId(a1.id))
 
+    def test_local_key_from_set_is_altered_and_saved(self):
+        a1 = LinkedAuthor(name='A1')
+        a1.save()
+        post = LinkedPost(title='P1', content='P2', authorId=a1.id)
+        post.save()
+        collection = Connection.get_collection(LinkedPost)
+        for item in collection.find({}):
+            self.assertEqual(item['authorId'], ObjectId(a1.id))
+        a2 = LinkedAuthor(name='A2')
+        post.set(authorId=a2.id)
+        post.save()
+        for item in collection.find({}):
+            self.assertEqual(item['authorId'], ObjectId(a2.id))
 
-# @jsonclass
-# class Customer(MongoObject):
-#     name: str
-#     products: List[Product] = types.listof('Product').linkedthru('customers')
+    def test_1l_1f_object_unlink_is_saved(self):
+        profile = LinkedProfile(name='p')
+        user = LinkedUser(name='u')
+        profile.user = user
+        profile.save()
+        profile.user = None
+        profile.save()
+        self.assertEqual(profile.is_new, False)
+        self.assertEqual(profile.is_modified, False)
+        self.assertEqual(profile.modified_fields, ())
+        self.assertEqual(user.is_new, False)
+        self.assertEqual(user.is_modified, False)
+        self.assertEqual(user.modified_fields, ())
+        collection = Connection.get_collection(LinkedProfile)
+        for item in collection.find({}):
+            self.assertEqual(item['userId'], None)
+
+    def test_1f_1l_object_unlink_is_saved(self):
+        user = LinkedUser(name='u')
+        profile = LinkedProfile(name='p')
+        user.profile = profile
+        user.save()
+        user.profile = None
+        user.save()
+        self.assertEqual(profile.is_new, False)
+        self.assertEqual(profile.is_modified, False)
+        self.assertEqual(profile.modified_fields, ())
+        self.assertEqual(user.is_new, False)
+        self.assertEqual(user.is_modified, False)
+        self.assertEqual(user.modified_fields, ())
+        collection = Connection.get_collection(LinkedProfile)
+        for item in collection.find({}):
+            self.assertEqual(item['userId'], None)
+
+    def test_1_many_unlink_is_saved(self):
+        author = LinkedAuthor(name='A')
+        post = LinkedPost(title='T', content='C')
+        author.posts.append(post)
+        author.save()
+        author.posts.remove(post)
+        author.save()
+        self.assertEqual(author.is_new, False)
+        self.assertEqual(author.is_modified, False)
+        self.assertEqual(author.modified_fields, ())
+        self.assertEqual(post.is_new, False)
+        self.assertEqual(post.is_modified, False)
+        self.assertEqual(post.modified_fields, ())
+        collection = Connection.get_collection(LinkedPost)
+        for item in collection.find({}):
+            self.assertEqual(item['authorId'], None)
+
+    def test_many_1_unlink_is_saved(self):
+        author = LinkedAuthor(name='A')
+        post = LinkedPost(title='T', content='C')
+        post.author = author
+        post.save()
+        post.author_id = None
+        post.save()
+        self.assertEqual(author.is_new, False)
+        self.assertEqual(author.is_modified, False)
+        self.assertEqual(author.modified_fields, ())
+        self.assertEqual(post.is_new, False)
+        self.assertEqual(post.is_modified, False)
+        self.assertEqual(post.modified_fields, ())
+        collection = Connection.get_collection(LinkedPost)
+        for item in collection.find({}):
+            self.assertEqual(item['authorId'], None)
+
+    def test_many_many_unlink_is_saved(self):
+        course1 = LinkedCourse(name='C1')
+        course2 = LinkedCourse(name='C2')
+        student1 = LinkedStudent(name='S1')
+        student2 = LinkedStudent(name='S2')
+        course1.students = [student1, student2]
+        course2.students = [student1, student2]
+        course1.save()
+        course1.students.remove(student1)
+        course1.save()
+        collname = 'linkedcoursesstudentslinkedstudentscourses'
+        collection = Connection('linked').collection(collname)
+        self.assertEqual(collection.count_documents({}), 3)
